﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System.Threading.Tasks;
using DocaAzure.Backup.Interfaces;
using DocaAzure.Storage;
using DocaAzure.Utils;
using NUnit.Framework;

namespace DocaAzure.Backup.Cmdlets.IntegrationTests
{
    [TestFixture]
    public class RestoreTableTests
    {
        public const string SampleData = "some data.";

        [SetUp]
        public void Setup()
        {
            AzureStorageAccountFactory.UseDevelopmentStorageAccount = true;

            CleanTestData();
        }

        [TearDown]
        public void TearDown()
        {
            CleanTestData();

            AzureStorageAccountFactory.UseDevelopmentStorageAccount = false;
        }

        [Test]
        [TestCase("mytable")]
        [TestCase("*")]
        [TestCase("")]
        [TestCase(null)]
        public void InitiatesTableRestore(string table)
        {
            Arrange();

            var cmd = new RestoreTable
            {
                Container = "cmdlet-test-container",
                Table = table,
                StorageAccountConnectionString = "UseDevelopmentStorage=true",
                MaxDegreeOfParallelism = 4
            };
 
            var e = cmd.Invoke().GetEnumerator();

            // this will actually call ProcessRecord() 
            e.MoveNext();

            var result = e.Current as Task<MonitoringResults>;

            Assert.IsNotNull(result);
            Assert.IsFalse(result.IsCompleted, "as the bus is mocked the messages won't be processed");

            // assert that the message(s) were sent
            Assert.IsNotNull(Utils.TryGetMessage());
        }

        private static void Arrange()
        {
            Utils.RecreateQueue();

            var provider = (IBlobOperations)new BlobStorageProvider();

            using (var sourceStream = SampleData.StringToStream())
            {
                sourceStream.ResetToBegining();
                provider.PutBlob("cmdlet-test-container/table1/08.to.10.1", sourceStream);

                sourceStream.ResetToBegining();
                provider.PutBlob("cmdlet-test-container/table2/08.to.10.1", sourceStream);
            }
        }

        private static void CleanTestData()
        {
            Utils.DeleteQueue();

            new BlobStorageProvider().DeleteContainer("cmdlet-test-container");
        }
    }
}
