﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System;
using Microsoft.WindowsAzure.StorageClient;

namespace DocaAzure.Storage.IntegrationTests
{
    #region sample classes

    public class TestBlobName : BlobName
    {
        public const string TestContainerName1 = "blob-test-container-1";

        public TestBlobName()
        {
            ContainerName = TestContainerName1;
        }

        public TestBlobName(string containerName)
        {
            ContainerName = containerName;
        }

        [BlobNamePart(0)]
        public string TestName { get; set; }
    }

    [Serializable]
    public class InnerTestSerializationClass
    {
        public string Property1 { get; set; }
        public int Property2 { get; set; }
    }

    [Serializable]
    public class TestSerializationClass
    {
        public InnerTestSerializationClass Property3 { get; set; }
        public Guid Property4 { get; set; }
    }

    #endregion sample classes

    public static class CommonTestUtils
    {
        #region sample data
        public const string SampleData =
"A user object receives a pointer to another object which implements an algorithm. The first is designated 'element class' and the latter 'the visitor class'." +
" The idea is to use a structure of element classes, each of which has an accept() method taking a visitor object for an argument. visitor is an interface having a visit()" +
" method for each element class. The accept() method of an element class calls back the visit() method for its class. Separate concrete visitor classes can then be written" +
" to perform some particular operations, by implementing these operations in their respective visit() methods." +
"One of these visit() methods of a concrete visitor can be thought of as a method not of a single class, but rather a method of a pair of classes: the concrete visitor" +
" and the particular element class. Thus the visitor pattern simulates double dispatch in a conventional single-dispatch object-oriented language such as Java, Smalltalk," +
" and C++. For an explanation of how double dispatch differs from function overloading, see Double dispatch is more than function overloading in the double dispatch article." +
" In the Java language, two techniques have been documented that use reflection to simplify the mechanics of double dispatch simulation in the visitor pattern: getting" +
" rid of accept() methods (the Walkabout variation), and getting rid of extra visit() methods." +
"The visitor pattern also specifies how iteration occurs over the object structure. In the simplest version, where each algorithm needs to iterate in the same way," +
" the accept() method of a container element, in addition to calling back the visit() method of the visitor, also passes the visitor object to the accept() method of all" +
" its constituent child elements." +
"Because the visitor object has one principal function (manifested in a plurality of specialized methods) and that function is called visit(), the visitor can be readily" +
" identified as a potential function object. Likewise, the accept() function can be identified as a function applicator, a mapper, which knows how to traverse a particular" +
" type of object and apply a function to its elements. Lisp's object system with its multiple dispatch does not replace the Visitor pattern, but merely provides a more" +
" concise implementation of it in which the pattern all but disappears.";
        #endregion sample data

        public static bool IsContainerWithPrivateBlobs(string containerName)
        {
            var account = AzureStorageAccountFactory.Create();
            var blobClient = new CloudBlobClient(account.BlobEndpoint, account.Credentials);

            // if container doesn't exists it will throw the exception
            var permissions = blobClient.GetContainerReference(containerName).GetPermissions();

            return permissions.PublicAccess == BlobContainerPublicAccessType.Off;
        }

        public static bool IsContainerExists(string containerName)
        {
            var account = AzureStorageAccountFactory.Create();
            var blobClient = new CloudBlobClient(account.BlobEndpoint, account.Credentials);

            // if container doesn't exists it will throw the exception
            try
            {
                blobClient.GetContainerReference(containerName).FetchAttributes();
            }
            catch (StorageClientException e)
            {
                if (IsResourceNotFoundError(e.ErrorCode))
                    return false;

                throw;
            }

            return true;
        }

        public static bool IsBlobExists(string blobName)
        {
            var account = AzureStorageAccountFactory.Create();
            var blobClient = new CloudBlobClient(account.BlobEndpoint, account.Credentials);

            // if container doesn't exists it will throw the exception
            try
            {
                blobClient.GetBlobReference(blobName).FetchAttributes();
            }
            catch (StorageClientException e)
            {
                if (IsResourceNotFoundError(e.ErrorCode))
                    return false;

                throw;
            }

            return true;
        }

        public static bool IsResourceNotFoundError(StorageErrorCode errorCode)
        {
            return (errorCode == StorageErrorCode.ContainerNotFound
                    || errorCode == StorageErrorCode.BlobNotFound
                    || errorCode == StorageErrorCode.ResourceNotFound);

        }

        public static void SafeDeleteContainer(CloudBlobClient blobClient, string containerName)
        {
            try
            {
                blobClient.GetContainerReference(containerName).Delete();
            }
            // ReSharper disable EmptyGeneralCatchClause
            catch
            // ReSharper restore EmptyGeneralCatchClause
            {
            }
        }
    }
}
