﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System.Data.Services.Client;
using DocaLabs.Storage.DataServicesUtils;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace DocaLabs.Storage.IntegrationTests.TestUtils
{
    public class AzureContextBase : DataServiceDbContext
    {
        public AzureContextBase()
            : base(new TableServiceContext(CloudStorageAccount.DevelopmentStorageAccount.TableEndpoint.ToString(), CloudStorageAccount.DevelopmentStorageAccount.Credentials), PartitionAndRowMappingMode.Discover)
        {
        }

        public override int SaveChanges()
        {
            ((TableServiceContext)Context).SaveChangesWithRetries(SaveChangesOptions.ReplaceOnUpdate);
            return -1;
        }

        public void CreateTablesIfNotExist()
        {
            var tableClient = CloudStorageAccount.DevelopmentStorageAccount.CreateCloudTableClient();

            foreach (var set in DiscoveredSets)
                tableClient.CreateTableIfNotExist(GetTableName(set));
        }

        public void DropTables()
        {
            var tableClient = CloudStorageAccount.DevelopmentStorageAccount.CreateCloudTableClient();

            foreach (var set in DiscoveredSets)
                tableClient.DeleteTableIfExist(GetTableName(set));
        }
    }
}
