﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System;
using DocaAzure.Backup.Impl;
using DocaAzure.Backup.Interfaces;

namespace DocaAzure.Backup
{
    public class BackupTableGraphBuilder
    {
        public BackupTableGraphBuilder BuilderParent { get; private set; }

        public TableBackup BackupNode { get; private set; }

        public TableBackup BackupRoot
        {
            get
            {
                var current = this;

                while (current.BuilderParent != null)
                    current = current.BuilderParent;

                return current.BackupNode;
            }
        }

        private BackupTableGraphBuilder(TableBackup node)
        {
            BackupNode = node;
        }

        public static BackupTableGraphBuilder CreateRoot(TableBackupBlobNameProvider blobNameProvider, string tableName, Func<BackupEntity, bool> entityFilter = null, IBackupStorageFactory writerFactory = null)
        {
            if (entityFilter == null)
                entityFilter = DefaultBackupServiceFactory.All();

            return new BackupTableGraphBuilder(new TableBackup(blobNameProvider, tableName, entityFilter, writerFactory));
        }

        public BackupTableGraphBuilder ChildByPartitionKey(string tableName, int maxDegreeOfParallelism = 1, IBackupStorageFactory writerFactory = null)
        {
            var childGraph = new TableBackup(BackupNode.BlobNameProvider, tableName, DefaultBackupServiceFactory.All(), writerFactory);

            BackupNode.AddChildTask(new TableBackupTask {TableBackup = childGraph, GetRanges = PartitionKeyExactMatchRange.FromPartitionKeyOfParent});

            return new BackupTableGraphBuilder(childGraph) { BuilderParent = this };
        }

        public BackupTableGraphBuilder SiblingByPartitionKey(string tableName, int maxDegreeOfParallelism = 1, IBackupStorageFactory writerFactory = null)
        {
            return BuilderParent == null
                       ? null
                       : BuilderParent.ChildByPartitionKey(tableName, maxDegreeOfParallelism, writerFactory);
        }

        public BackupTableGraphBuilder ChildByRowKey(string tableName, int maxDegreeOfParallelism = 1, IBackupStorageFactory writerFactory = null)
        {
            var childGraph = new TableBackup(BackupNode.BlobNameProvider, tableName, DefaultBackupServiceFactory.All(), writerFactory);

            BackupNode.AddChildTask(new TableBackupTask { TableBackup = childGraph, GetRanges = PartitionKeyExactMatchRange.FromRowKeyOfParent });

            return new BackupTableGraphBuilder(childGraph) { BuilderParent = this };
        }

        public BackupTableGraphBuilder SiblingByRowKey(string tableName, int maxDegreeOfParallelism = 1, IBackupStorageFactory writerFactory = null)
        {
            return BuilderParent == null
                       ? null
                       : BuilderParent.ChildByRowKey(tableName, maxDegreeOfParallelism, writerFactory);
        }
    }
}
