﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System;
using System.Threading.Tasks;
using DocaAzure.Backup.Impl;
using DocaAzure.Storage;
using Microsoft.WindowsAzure;

namespace DocaAzure.Backup
{
    public class BlobContainerBackup : BlobContainerBackupRestoreBase
    {
        public void Backup(string[] sourceContainerNames, string destinationContainerName, DateTime? onlyIfModifiedSince, int maxDegreeOfParallelism = 1)
        {
            if (!destinationContainerName.EndsWith("/"))
                destinationContainerName += "/";

            MakeDestinationBlobName = blobName => destinationContainerName + blobName;

            var provider = new BlobStorageProvider(GetSourceAccount());

            new BlobStorageProvider(GetDestinationAccount()).CreateContainer(destinationContainerName);

            var filter = onlyIfModifiedSince == null
                                                ? null
                                                : BlobStorageProvider.IfModifiedSince(onlyIfModifiedSince.GetValueOrDefault());

            foreach (var sourceContainerName in sourceContainerNames)
            {
                Parallel.ForEach(
                            provider.FlatBlobList(sourceContainerName, null, filter),
                            new ParallelOptions { MaxDegreeOfParallelism = maxDegreeOfParallelism },
                            GetCopyAction());
            }
        }

        protected override CloudStorageAccount GetSourceAccount()
        {
            return AzureStorageAccountFactory.Create();
        }

        protected override CloudStorageAccount GetDestinationAccount()
        {
            return DefaultBackupServiceFactory.Default.GetStorageAccount();
        }
    }
}
