﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System.Xml;
using System.Xml.Linq;
using DocaAzure.Storage;

namespace DocaAzure.Backup.Impl
{
    /// <summary>
    /// Entities used for backup/restore
    /// </summary>
    public class BackupEntity : XmlTableServiceEntity
    {
        private int _approxEntitySize = -42;

        /// <summary>
        /// Used during restore to store the entry element.
        /// </summary>
        public override XElement EntryElement
        {
            set
            {
                base.EntryElement = value;
                _approxEntitySize = -42;
            }
        }

        public int RecalculateApproxEntitySize()
        {
            _approxEntitySize = -42;

            return GetApproxEntitySize();
        }

        public int GetApproxEntitySize()
        {
            if (EntryElement == null)
                return 0;

            if (_approxEntitySize == -42)
                _approxEntitySize = EntryElement.ToString().Length * 2;

            return _approxEntitySize;
        }

        public void WriteTo(XmlWriter writer)
        {
            EntryElement.WriteTo(writer);
        }
    }
}
