﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System.Collections.Generic;

namespace DocaAzure.Backup.Impl
{
    /// <summary>
    /// Represents all blocks in a blob
    /// </summary>
    public class Blob
    {
        public const int DefaultMaxBlocksInBlobs = 100;

        private readonly int _maxBlocksInBlobs;

        private readonly State _state;

        public Blob(State state, int maxBlocksInBlobs)
        {
            _state = state;
            _maxBlocksInBlobs = maxBlocksInBlobs;
        }

        /// <summary>
        /// The blocks that form the blob
        /// </summary>
        public IEnumerable<Block> Blocks
        {
            get
            {
                var blockCount = 0;

                while (!_state.HasCompleted && blockCount < _maxBlocksInBlobs)
                {
                    blockCount++;
                    yield return new Block(_state);
                }
            }
        }
    }
}
