﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System;
using System.Collections.Generic;

namespace DocaAzure.Backup.Impl
{
    /// <summary>
    /// Represents all batches in a block
    /// </summary>
    public class Block
    {
        public const int DefaultMaxBlockSize = (int)(3.8 * 1024 * 1024);

        private readonly State _state;

        public string BlockId { get; private set; }

        public Block(State state)
        {
            _state = state;
            BlockId = Convert.ToBase64String(Guid.NewGuid().ToByteArray());
        }

        /// <summary>
        /// The list of batches in the block. 
        /// </summary>
        public IEnumerable<Batch> Batches
        {
            get
            {
                while (!_state.HasCompleted && _state.CurrentBlockSize < _state.MaxBlockSize)
                {
                    // check that there is enough space left in the block for at least one entity (in the next batch)
                    if(_state.LookAheadEntity != null)
                    {
                        var approxEntitySize = _state.LookAheadEntity.GetApproxEntitySize();
                        if (_state.CurrentBlockSize + approxEntitySize > _state.MaxBlockSize)
                            yield break;
                    }

                    yield return new Batch(_state);
                }
            }
        }
    }
}
