﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System;
using System.IO;
using DocaAzure.Backup.Interfaces;
using DocaAzure.Utils;

namespace DocaAzure.Backup.Impl
{
    public class UncompressedBuffer : IBuffer
    {
        private MemoryStream _buffer;

        public int MaxBlockSize { get; set; }

        public long ApproxLength
        {
            get
            {
                if (_buffer == null)
                    return 0;

                _buffer.Flush();

                return _buffer.Position;
            }
        }

        public Stream DataStream { get { return _buffer; } }

        public IDisposable StartNewBlock()
        {
            if (_buffer == null)
            {
                _buffer = new MemoryStream((int)(1.1 * MaxBlockSize));  // extra 10%
            }
            else
            {
                _buffer.ResetToBegining();
                _buffer.SetLength(0);
            }

            return new DisposableDummy();
        }

        public Stream GetFinalData()
        {
            if (_buffer == null)
                return null;

            _buffer.SetLength(_buffer.Position);

            return _buffer.ResetToBegining();
        }

        public Stream InflateStream(Stream stream)
        {
            return stream;
        }

        public void Dispose()
        {
            ClearBufferStream();
        }

        private void ClearBufferStream()
        {
            if (_buffer == null)
                return;

            _buffer.Dispose();
            _buffer = null;
        }

        class DisposableDummy : IDisposable
        {
            public void Dispose()
            {
            }
        }
    }
}
