﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System.Collections.Generic;

namespace DocaAzure.Backup.Interfaces
{
    /// <summary>
    /// Defines interface that provides objects 
    /// to write into backup storage during backup
    /// or read from the backup storage during restore
    /// </summary>
    public interface IBackupStorageFactory
    {
        /// <summary>
        /// Creates table backup writer
        /// </summary>
        /// <param name="blobName">Blob name</param>
        /// <returns>Table backup writer</returns>
        ITableBackupWriter GetTableBackupWriter(string blobName);

        /// <summary>
        /// Creates table restore reader
        /// </summary>
        /// <param name="blobName">Blob name</param>
        /// <returns>FTable restore reader</returns>
        ITableRestoreReader GetTableRestoreReader(string blobName);

        /// <summary>
        /// Returns detected blob names for specified table
        /// </summary>
        /// <param name="backupContainerName">Container where to look for</param>
        /// <param name="tablePrefix">Table prefix - includes whatever path to the table and table name itself</param>
        /// <returns>List of blobs</returns>
        IEnumerable<string> GetTableBlobs(string backupContainerName, string tablePrefix);

        /// <summary>
        /// Returns detected table names in the specified backup container
        /// </summary>
        /// <param name="backupContainerName">Container where to look for</param>
        /// <returns>List of tables</returns>
        IEnumerable<string> GetTables(string backupContainerName);

        /// <summary>
        /// Returns detected original container names in the specified backup container
        /// </summary>
        /// <param name="backupContainerName">Container where to look for</param>
        /// <returns>List of containers</returns>
        IEnumerable<string> GetOriginalContainers(string backupContainerName);
    }
}
