﻿#region License
/*
 * Copyright ©2010-2011 Doca Labs Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using DocaAzure.Backup.Impl;
using DocaAzure.Backup.Interfaces;
using Newtonsoft.Json;

namespace DocaAzure.Backup
{
    [Serializable]
    public class PartitionKeyExactMatchRange : ITableRange
    {
        public string PartitionKey { get; private set; }

        [JsonConstructor]
        public PartitionKeyExactMatchRange(string partitionKey)
        {
            PartitionKey = partitionKey;
        }

        public static IEnumerable<ITableRange> FromPartitionKeyOfParent(BackupEntity parent)
        {
            return new[] { new PartitionKeyExactMatchRange(parent.PartitionKey) };
        }

        public static IEnumerable<ITableRange> FromRowKeyOfParent(BackupEntity parent)
        {
            return new[] { new PartitionKeyExactMatchRange(parent.RowKey) };
        }

        public static IEnumerable<ITableRange> FromPartitionKeyValue(string partitionKey)
        {
            return new[] { new PartitionKeyExactMatchRange(partitionKey) };
        }

        public IQueryable ApplyRange(IQueryable input)
        {
            var query = (IQueryable<BackupEntity>)input;

            if (!string.IsNullOrWhiteSpace(PartitionKey))
                query = query.Where(entity => entity.PartitionKey == PartitionKey);

            return query;
        }

        public override string ToString()
        {
            return HttpUtility.UrlEncode((PartitionKey ?? "null"));
        }
    }
}
